;Machine code routines to add a cursor on/off facility and case 
;converter
;Syntax: CURSON n       turn on cursor in window #n  
;        CURSOF n       turn off cursor in window #n  
;        CONV$(string)  returns string all in upper case
;
; Symbol definitions:  must precede use
;
ERROR_BP        EQU     -15
CH_LENCH        EQU     $28
BV_RIP          EQU     $58
BV_CHBAS        EQU     $30
BP_INIT         EQU     $110
CA_GTINT        EQU     $112
CA_GTSTR        EQU     $116
CA_GTLIN        EQU     $118
BV_CHRIX        EQU     $11A
SD_CURE         EQU     $E
SD_CURS         EQU     $F
;
;Initialisation routine
;
ADD_FUN
        LEA     PROC_DEF,A1         ; pointer to definition table
        MOVE    BP_INIT,A2          ; initialise vector
        JSR     (A2)                ; do it
        RTS                         ; back to basic
;
;Procedure definition table
;
PROC_DEF
        DC.W    2                    ; number of procedures - 2
        DC.W    CON_OF-*             ; w.offset to 1st proc
        DC.B    @2-@1
@1      DC.B    'CURSON'
@2
        ALIGN
        DC.W    COFF_OF-*            ; W.Offset to 2nd proc
        DC.B    @4-@3
@3      DC.B    'CURSOF'
@4
        ALIGN                        ; even boundary
        DC.W    0                    ; end of proc. definitions
        DC.W    1                    ; number of functions - 1
        DC.W    CONV_OF-*            ; word length offset to first fn
        DC.B    @6-@5                ; name length
@5      DC.B    'CONV$'              ; actual name
@6
        ALIGN                        ; ensure a word boundary
        DC.W    0                    ; end of fn. definitions
;
CONV_OF        
        move    CA_GTSTR,a2
        jsr     (a2)                 ; get string argument from BASIC
        bne.s   EXIT                 ; error return
        subq    #1,d3                ; just one parameter?
        bne.s   err_bp               ; error if not
        
        move.w  0(a6,a1.l),d7        ; get length of string
        movea.l  a1,a0               ; hold copy of a1 in d0
        addq.l  #2,a0                ; increment pointer
SCAN_START
        tst.w   d7                   ; at end of string ?
        beq.s   SET_STACK
        subq.w  #1,d7                ; process another character
        
        move.b  0(a6,a0.l),d1         ; get char
              
        cmpi.b  #'a',d1
        blo.s   NO_CONV
        cmpi.b  #'z', d1
        bhi.s   NO_CONV
; is lower case so convert
        subi.b  #'a'-'A', 0(a6,a0.l)        
NO_CONV
        addq.l  #1,a0                 ;increment stack pointer 
        bra.s   SCAN_START

SET_STACK
        moveq   #1,d4                ; tell BASIC STRING type returned
        move.l  a1,BV_RIP(a6)        ; write RI pointer to Basic variable
        clr.l   d0                   ; no errors
        rts                          ; back to BASIC
        
;
ERR_BP
        moveq   #ERROR_BP,d0         ; bad parameter
EXIT
        rts
        
;********************************
;
; CURSON ON/OFF ROUTINES...
;
;********************************

CON_OF
        move.b  #SD_CURE,d5          ; cursor enable
        bra.s   CURS_START
COFF_OF
        move.b  #SD_CURS,d5          ; cursor disable
CURS_START
        moveq   #1,d4            ; default channel id
        movea   CA_GTINT,a2      ; also puts stack ptr into a1
        jsr     (a2)
        bne.s   EXIT             ; ..error
        subq.w  #1,d3            ; check only 1 param
        bne.s   ERR_BP           ; ..error (bad param)
        move.w  0(a6,a1.l),d4    ; put channel no. in d4
        addq.l  #2,a1            ; unstack
        move.l  a1,BV_RIP(a6)    ; replace stack value
        mulu    #CH_LENCH,d4     ; look up channel table
        add.l   BV_CHBAS(a6),d4  ; d4 now contains channel id
        move.l  0(a6,d4.l),a0    ; extract channel id
        move.l  #500,d3          ; timeout
        move.b  d5,d0            ; operation to perform on/off
        trap    #3
        rts

        end                      ; of assembly
